@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------------------------------------------------
-# scripts/btContainerApps.py is part of Brewtarget, and is copyright the following authors 2025:
+# scripts/btContainerApps.py is part of Brewtarget, and is copyright the following authors 2025-2026:
 #   • Matt Young <mfsy@yahoo.com>
 #
 # Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
@@ -137,13 +137,15 @@ def doAppImage():
    #
    #    [projectName]-[version]   <── dir_appDir
    #    ├── AppRun ❇   <── Shell script that acts as the entry point for the AppImage
-   #    ├── [projectName].desktop     ❇
+   #    ├── [reverseDomain].[projectName].desktop = SYMLINK to file in share/applications  ❇
    #    └── usr
    #        ├── bin
    #        │   └── [projectName] ✅   <── the executable
-   #        ├── lib
-   #        │   └── [All the libraries needed by the application]
+   #        ├── lib   <── dir_appDir_lib
+   #        │   └── [All the libraries needed by the application] ❇
    #        └── share
+   #            ├── applications   <── dir_applications
+   #            │   └── [reverseDomain].[projectName].desktop ✅
    #            ├── [projectName]
    #            │   ├── DefaultData.xml           ✅  <──┬── [filesToInstall_data]
    #            │   ├── default_db.sqlite         ✅  <──┘
@@ -151,29 +153,43 @@ def doAppImage():
    #            │   │   └── [All the filesToInstall_sounds .wav files] ✅
    #            │   └── translations_qm
    #            │       └── [All the .qm files generated by qt.compile_translations] ✅
-   #            └── icons
-   #                └── hicolor
-   #                    └── scalable
-   #                        └── apps
-   #                            └── [projectName].svg ✅  <── [filesToInstall_icons]
+   #            ├── icons   <── dir_icons
+   #            │   └── hicolor
+   #            │       └── scalable
+   #            │           └── apps
+   #            │               └── [projectName].svg ✅  <── [filesToInstall_icons]
+   #            └── metainfo   <── dir_metainfo
+   #                └── [reverseDomain].[projectName].appdata.xml ❇
    #
 
-   # Move [projectName].desktop to the correct place for AppImage
+   # Make a symlink to the .desktop file in the AppImage root folder
    dir_applications = dir_appDir.joinpath('usr').joinpath('share').joinpath('applications')
-   shutil.move(dir_applications.joinpath(appId + '.desktop'),
-               dir_appDir.as_posix())
-   os.rmdir(dir_applications)
+   os.symlink(dir_applications.relative_to(dir_appDir).joinpath(appId + '.desktop'),
+              dir_appDir.joinpath(appId + '.desktop'))
    # Move [projectName].svg to the correct place for AppImage
    dir_icons = dir_appDir.joinpath('usr').joinpath('share').joinpath('icons')
    shutil.move(dir_icons.joinpath('hicolor').joinpath('scalable').joinpath('apps').joinpath(projectName + '.svg'),
                dir_appDir.as_posix())
    shutil.rmtree(dir_icons)
 
+   #
+   # One of the warning messages from the AppImage generator is "AppStream upstream metadata is missing" if it can't
+   # find usr/share/metainfo/[reverseDomain].appdata.xml.  It points you to more information at
+   # https://www.freedesktop.org/software/appstream/docs/chap-Quickstart.html.  The page at that link tells you the
+   # metadata file should be called /usr/share/metainfo/%{id}.metainfo.xml (where %{id} follows a reverse-DNS scheme,
+   # consisting of {tld}.{vendor}.{product}.
+   #
+   # TLDR is we need to rename [reverseDomain].metainfo.xml to [reverseDomain].appdata.xml
+   #
+   dir_metainfo = dir_appDir.joinpath('usr').joinpath('share').joinpath('metainfo')
+   shutil.move(dir_metainfo.joinpath(appId + '.metainfo.xml'),
+               dir_metainfo.joinpath(appId + '.appdata.xml'))
+
    # Create the AppRun shell script...
    file_appRun = dir_appDir.joinpath('AppRun')
    with open(file_appRun, 'w') as appRunFile:
-      appRunFile.write('#!/bin/bash')
-      appRunFile.write('exec $APPDIR/usr/bin/' + projectName)
+      appRunFile.write('#!/bin/bash\n')
+      appRunFile.write('exec $APPDIR/usr/bin/' + projectName + '\n')
    # ...and make it executable
    stat_appRun = os.stat(file_appRun)
    os.chmod(file_appRun, stat_appRun.st_mode | stat.S_IEXEC)
@@ -201,6 +217,7 @@ def doAppImage():
    #
    #
    dir_appDir_lib = dir_appDir.joinpath('usr').joinpath('lib')
+   os.makedirs(dir_appDir_lib)
    for lddOutputLine in lddOutput.splitlines():
       fields = lddOutputLine.strip().split()
       if len(fields) == 4:
@@ -209,6 +226,8 @@ def doAppImage():
          shutil.copy2(libPath, dir_appDir_lib)
       else:
          btLogger.log.debug('Skipping "' + lddOutputLine + '"')
+   btLogger.log.info('Libraries to ship (in ' + dir_appDir_lib.as_posix() + '):')
+   os.listdir(dir_appDir_lib.as_posix())
 
    #
    # Now we run the AppImage tool to turn the directory tree into an app image.  Note that we first have to set the ARCH
@@ -756,4 +775,4 @@ def doSnap():
 
    # TODO - Finish this!
 
-   return
\ No newline at end of file
+   return
