@@ -1,5 +1,5 @@
 /*╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
- * qtModels/tableModels/TableModelBase.h is part of Brewtarget, and is copyright the following authors 2023-2025:
+ * qtModels/tableModels/TableModelBase.h is part of Brewtarget, and is copyright the following authors 2023-2026:
  *   • Matt Young <mfsy@yahoo.com>
  *
  * Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
@@ -29,36 +29,34 @@
 #include "undoRedo/Undoable.h"
 #include "measurement/Measurement.h"
 #include "model/IngredientAmount.h"
-#include "model/IngredientInRecipe.h"
-#include "model/Inventory.h"
+#include "model/StockPurchase.h"
 #include "model/Recipe.h"
 #include "qtModels/tableModels/BtTableModel.h"
 #include "undoRedo/UndoableAddOrRemove.h"
+#include "utils/ColumnOwnerTraits.h"
 #include "utils/CuriouslyRecurringTemplateBase.h"
 #include "utils/MetaTypes.h"
 #include "utils/PropertyHelper.h"
+#include "utils/TypeTraits.h"
 
 // TODO: We would like to change "Add to Recipe" to "Set for Recipe" for things where the recipe only has one of them, eg Style or Equipment
 
 class Style;
 
 //
-// Using concepts allows us to tailor the templated TableModelTraits class without having the same inheritance
+// Using concepts allows us to tailor the templated TableModelBase class without having the same inheritance
 // structure as BtTableModel / BtTableModelRecipeObserver.
 //
 // Note that concepts have some limitations:
 //    - Concepts cannot recursively refer to themselves and cannot be constrained, is you cannot define one concept in
 //      terms of another
 //    - You cannot have explicit instantiations, explicit specializations, or partial specializations of concepts
-// An alternative would be to use a traits struct, which has fewer limitations, but is somewhat more clunky.
 //
 // See comment in utils/TypeTraits.h for definition of CONCEPT_FIX_UP (and why, for now, we need it)
 template <typename T> concept CONCEPT_FIX_UP IsTableModel         = std::is_base_of_v<BtTableModel, T>;
 template <typename T> concept CONCEPT_FIX_UP ObservesRecipe       = std::is_base_of_v<BtTableModelRecipeObserver, T>;
 template <typename T> concept CONCEPT_FIX_UP DoesNotObserveRecipe = std::negation_v<std::is_base_of<BtTableModelRecipeObserver, T>>;
 
-template <typename T> concept CONCEPT_FIX_UP IsIngredientInRecipe = std::is_base_of_v<IngredientInRecipe, T>;
-
 //
 // NOTE: At several places below in TableModelBase, we would like to have two versions of a member function depending on
 //       some property of Derived - eg using `requires HasSomeAttribute<Derived>`.  Although GCC lets you do this, other
@@ -71,9 +69,9 @@ template <typename T> concept CONCEPT_FIX_UP IsIngredientInRecipe = std::is_base
 //       The way round this is to template the member function so that the evaluation of the constraint is deferred
 //       until after the class declaration of `Derived` is complete.  Now, in, eg, HopTableModel we can call the right
 //       version via:
-//          this->updateInventory<HopTableModel>(...);
+//          this->updateStockPurchase<HopTableModel>(...);
 //
-//       (I did also try `this->updateInventory<decltype(*this)>(...)`, but I couldn't get it to work.
+//       (I did also try `this->updateStockPurchase<decltype(*this)>(...)`, but I couldn't get it to work.)
 //
 //       But what if we want to call one of these templated functions from inside TableModelBase?  Well, then we have to
 //       call down to a wrapper function in Derived that knows which version of the TableModelBase function to call.  Eg
@@ -95,18 +93,12 @@ template <typename T> concept CONCEPT_FIX_UP IsIngredientInRecipe = std::is_base
 //       what is going on.
 //
 
-
-/**
- * \brief We want, eg, \c HopTableModel to inherit from \c TableModelBase<HopTableModel, Hop> and to have its own enum
- *        \c HopTableModel::ColumnIndex.  But we'd also like \c HopTableModel::ColumnIndex to be accessible from within
- *        \c TableModelBase, which normally isn't possible, eg as explained at
- *        https://stackoverflow.com/questions/5534759/c-with-crtp-class-defined-in-the-derived-class-is-not-accessible-in-the-base
- *        However, per the same link, the way around this is to use a traits class.  This is another "trick" where we
- *        declare a template for the "traits" class before the base class of the curiously recurring template pattern
- *        (CRTP), but then specialise that "traits" class in the derived class.
- */
-template<class Derived>
-struct TableModelTraits;
+//
+// This is currently the least clunky way for determining at compile-time whether a particular enum member exists
+//
+template <typename E> concept HasMemberName           = requires { E::Name          ; };
+template <typename E> concept HasMemberTotalInventory = requires { E::TotalInventory; };
+template <typename E> concept HasMemberPctAcid        = requires { E::PctAcid       ; };
 
 /**
  * \brief See comment in qtModels/tableModels/BtTableModel.h for more info on inheritance structure
@@ -122,17 +114,32 @@ struct TableModelTraits;
  *                                                    // calls to removed() because avoids rounding errors on running
  *                                                    // totals.)
  *
- *        Note that we use TableModelTraits as the phantom class for CuriouslyRecurringTemplateBase since it fits the
+ *        Note that we use ColumnOwnerTraits as the phantom class for CuriouslyRecurringTemplateBase since it fits the
  *        bill.
  */
 template<class Derived, class NE>
-class TableModelBase : public CuriouslyRecurringTemplateBase<TableModelTraits, Derived> {
+class TableModelBase : public CuriouslyRecurringTemplateBase<ColumnOwnerTraits, Derived> {
 public:
+   //
+   // We want, eg, \c HopTableModel to inherit from \c TableModelBase<HopTableModel, Hop> and to have its own enum
+   // \c HopTableModel::ColumnIndex.  But we'd also like \c HopTableModel::ColumnIndex to be accessible from within
+   // \c TableModelBase, which normally isn't possible, eg as explained at
+   // https://stackoverflow.com/questions/5534759/c-with-crtp-class-defined-in-the-derived-class-is-not-accessible-in-the-base
+   // However, per the same link, the way around this is to use a traits class.  This is another "trick" where we
+   // declare a template for the "traits" class before the base class of the curiously recurring template pattern
+   // (CRTP), but then specialise that "traits" class in the derived class.
+   //
    // This gets round the fact that we would not be able to access Derived::ColumnIndex directly
    //
-   // In theory, in C++20, we don't need the `typename` here, but, per comment in BtTableModel::ColumnInfo, we need to
+   // In theory, in C++20, we don't need the `typename` here, but, per comment in ColumnInfo, we need to
    // retain it until our Mac build environment is using a more recent version of Clang.
-   using ColumnIndex = typename TableModelTraits<Derived>::ColumnIndex;
+   //
+   using ColumnIndex = typename ColumnIndexHolder<Derived>::ColumnIndex;
+
+   //
+   // This saves us having to have NE as a template parameter for SortFilterProxyModelBase
+   //
+   using UnderlyingItem = NE;
 
 protected:
    TableModelBase() : m_rows{} {
@@ -150,14 +157,14 @@ public:
     *        resolution (as explained at
     *        https://stackoverflow.com/questions/51690394/overloading-member-function-among-multiple-base-classes).
     */
-   BtTableModel::ColumnInfo const & get_ColumnInfo(ColumnIndex const columnIndex) const {
-      return this->derived().BtTableModel::getColumnInfo(static_cast<size_t>(columnIndex));
+   ColumnInfo const & get_ColumnInfo(ColumnIndex const columnIndex) const {
+      return ColumnOwnerTraits<Derived>::getColumnInfo(static_cast<size_t>(columnIndex));
    }
 
    /**
     * \brief Overload for \c get_ColumnInfo
     */
-   BtTableModel::ColumnInfo const & get_ColumnInfo(QModelIndex const & index) const {
+   ColumnInfo const & get_ColumnInfo(QModelIndex const & index) const {
       auto const columnIndex = static_cast<ColumnIndex>(index.column());
       return this->get_ColumnInfo(columnIndex);
    }
@@ -352,7 +359,7 @@ public:
     * \brief Use this for removing \c RecipeAdditionHop etc
     */
    template<class Proxy>
-   void removeSelectedIngredients(QTableView & tableView, Proxy & proxy) /*requires IsIngredientInRecipe<Derived>*/ {
+   void removeSelectedIngredients(QTableView & tableView, Proxy & proxy) {
       QModelIndexList selected = tableView.selectionModel()->selectedIndexes();
       QList<std::shared_ptr<NE>> itemsToRemove;
 
@@ -393,76 +400,8 @@ public:
       QVariant  leftItem = this->readDataFromModel( leftIndex, Qt::UserRole);
       QVariant rightItem = this->readDataFromModel(rightIndex, Qt::UserRole);
 
-      // Normally leave this commented out as it generates far too much logging
-//      qDebug() << Q_FUNC_INFO << "leftItem:" << leftItem << "; rightItem:" << rightItem;
-
-      //
-      // It's not crazy to have null come before valid values
-      //
-      if (leftItem.isNull()) {
-         return true;
-      }
-      if (rightItem.isNull()) {
-         return false;
-      }
-
-      BtTableModel::ColumnInfo const & columnInfo = this->get_ColumnInfo(leftIndex);
-      if (columnInfo.typeInfo.fieldType) {
-         QuantityFieldType const fieldType = *columnInfo.typeInfo.fieldType;
-         if (std::holds_alternative<NonPhysicalQuantity>(fieldType)) {
-            auto const nonPhysicalQuantity = std::get<NonPhysicalQuantity>(fieldType);
-            switch (nonPhysicalQuantity) {
-               case NonPhysicalQuantity::Date:
-               case NonPhysicalQuantity::String:
-               case NonPhysicalQuantity::Enum:
-               case NonPhysicalQuantity::Bool:
-                  return leftItem.toString() < rightItem.toString();
-
-               case NonPhysicalQuantity::Percentage:
-               case NonPhysicalQuantity::Dimensionless:
-                  return leftItem.toDouble() < rightItem.toDouble();
-
-               case NonPhysicalQuantity::OrdinalNumeral:
-               case NonPhysicalQuantity::CardinalNumber:
-                  return leftItem.toInt() < rightItem.toInt();
-
-               // No default case as we want compiler to warn us if we missed a case above
-            }
-         } else if (std::holds_alternative<Measurement::PhysicalQuantity>(fieldType)) {
-            return  leftItem.value<Measurement::Amount>().toCanonical() <
-                   rightItem.value<Measurement::Amount>().toCanonical();
-         } else {
-            Q_ASSERT(std::holds_alternative<Measurement::ChoiceOfPhysicalQuantity>(fieldType));
-            //
-            // Per comment in readDataFromModel(), this field could be either the amount itself or a drop-down for the
-            // PhysicalQuantity of the amount.  In the latter case, readDataFromModel() will have returned us a QString,
-            // otherwise we'll have an Amount.
-            //
-            if (leftItem.typeId() == QMetaType::QString) {
-               //
-               // Field type should be determined only by column, not by row
-               //
-               Q_ASSERT(rightItem.typeId() == QMetaType::QString);
-               return leftItem.toString() < rightItem.toString();
-            }
-
-            //
-            // It's not instantly obvious how to sort a mixture of, eg, masses and volumes.  For the moment, we convert
-            // everything to canonical units (kilograms for mass, liters for volume, etc) and then sort by the
-            // quantities.
-            //
-            return  leftItem.value<Measurement::Amount>().toCanonical() <
-                   rightItem.value<Measurement::Amount>().toCanonical();
-         }
-
-         //
-         // It's a coding error if we didn't handle every case above
-         //
-         qWarning() << Q_FUNC_INFO << "Unhandled field type:" << fieldType;
-      }
-
-      return true;
-
+      ColumnInfo const & columnInfo = this->get_ColumnInfo(leftIndex);
+      return PropertyHelper::isLessThan(leftItem, rightItem, columnInfo.typeInfo);
    }
 
 protected:
@@ -567,7 +506,9 @@ protected:
          return false;
       }
 
-      if (role != Qt::DisplayRole && role != Qt::EditRole) {
+      if (role != Qt::DisplayRole &&
+          role != Qt::EditRole &&
+          role != Qt::TextAlignmentRole) {
          // No need to log anything here, as it's perfectly normal to get called with other roles
          return false;
       }
@@ -600,7 +541,11 @@ protected:
       //    QAbstractItemView::edit()
       //
       auto row = this->m_rows[index.row()];
-      BtTableModel::ColumnInfo const & columnInfo = this->get_ColumnInfo(index);
+      ColumnInfo const & columnInfo = this->get_ColumnInfo(index);
+      TypeInfo const & typeInfo = columnInfo.typeInfo;
+      if (role == Qt::TextAlignmentRole) {
+         return PropertyHelper::getAlignment(typeInfo);
+      }
 
       QVariant modelData = columnInfo.propertyPath.getValue(*row);
       if (!modelData.isValid()) {
@@ -611,10 +556,7 @@ protected:
          Q_ASSERT(false); // Stop here on debug builds
       }
 
-      TypeInfo const & typeInfo = columnInfo.typeInfo;
-
-
-      // Uncomment this log statement if asserts below are firing
+      // Uncomment this log statement if asserts in PropertyHelper::readDataFromPropertyValue are firing
 //      qDebug() <<
 //         Q_FUNC_INFO << columnInfo.columnFqName << ", propertyPath:" << columnInfo.propertyPath << "TypeInfo:" <<
 //         typeInfo << ", modelData:" << modelData;
@@ -673,29 +615,34 @@ protected:
    }
 
    template<class TM>
-   void updateInventory(int invKey, BtStringConst const & propertyName) requires IsTableModel<TM> &&
-                                                                                 CanHaveInventory<NE> {
+   void updateStockPurchase(int invKey,
+                            BtStringConst const & propertyName) requires IsTableModel<TM> && CanHaveStockPurchase<NE> {
+      //
       // Substantive version
+      //
+      // Find the StockPurchase that we've been notified about; check whether any of our rows relate to the same
+      // ingredient; if so, send out a signal that the "total inventory" column for that row needs updating.
+      //
+      // TBD: I think we need to revisit this to ensure it workes properly for StockUse and for RecipeAddition tables.
+      //
       if (propertyName == PropertyNames::IngredientAmount::amount) {
+         auto stockPurchase = ObjectStoreWrapper::getById<typename NE::StockPurchaseClass>(invKey);
          for (int ii = 0; ii < this->m_rows.size(); ++ii) {
             std::shared_ptr<NE> ingredient = this->m_rows.at(ii);
-            if (InventoryTools::hasInventory<NE>(*ingredient)) {
-               std::shared_ptr<typename NE::InventoryClass> inventory = InventoryTools::getInventory(*ingredient);
-               if (inventory->key() == invKey) {
-                  emit this->derived().dataChanged(
-                     this->derived().createIndex(ii, static_cast<int>(Derived::ColumnIndex::TotalInventory)),
-                     this->derived().createIndex(ii, static_cast<int>(Derived::ColumnIndex::TotalInventory))
-                  );
-               }
+            if (ingredient->key() == stockPurchase->ingredientId()) {
+               emit this->derived().dataChanged(
+                  this->derived().createIndex(ii, static_cast<int>(Derived::ColumnIndex::TotalInventory)),
+                  this->derived().createIndex(ii, static_cast<int>(Derived::ColumnIndex::TotalInventory))
+               );
             }
          }
       }
       return;
    }
    template<class TM>
-   void updateInventory([[maybe_unused]] int invKey,
+   void updateStockPurchase([[maybe_unused]] int invKey,
                         [[maybe_unused]] BtStringConst const & propertyName) requires IsTableModel<TM> &&
-                                                                                      CannotHaveInventory<NE> {
+                                                                                      CannotHaveStockPurchase<NE> {
       // No-op version
       return;
    }
@@ -775,6 +722,72 @@ protected:
       return this->readDataFromModel(index, role);
    }
 
+   Qt::ItemFlags doFlags(QModelIndex const & index) const {
+      //
+      // Per https://doc.qt.io/qt-6/qt.html#ItemFlag-enum, Qt::ItemFlag is a combination of one or more of the
+      // following:
+      //
+      //    Qt::NoItemFlags          - No properties set.
+      //    Qt::ItemIsSelectable     - Can be selected.
+      //    Qt::ItemIsEditable       - Can be edited.
+      //    Qt::ItemIsDragEnabled    - Can be dragged.
+      //    Qt::ItemIsDropEnabled    - Can be used as a drop target.
+      //    Qt::ItemIsUserCheckable  - Can be checked or unchecked by the user.
+      //    Qt::ItemIsEnabled        - User can interact with the item.
+      //    Qt::ItemIsAutoTristate   - Item's state depends on state of its children.  Enables automatic management of
+      //                               state of parent items in QTreeWidget (checked if all children are checked,
+      //                               unchecked if all children are unchecked, or partially checked if only some
+      //                               children are checked).
+      //    Qt::ItemNeverHasChildren - Item never has child items.  (Used for optimization purposes only.)
+      //    Qt::ItemIsUserTristate   - User can cycle through three separate states.
+      //
+      bool const tableIsEditable = this->derived().m_editable;
+      static Qt::ItemFlags const defaults = Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled;
+      auto const columnIndex = static_cast<ColumnIndex>(index.column());
+
+      // Normally leave this log statement commented out to avoid generating too much logging
+      ColumnInfo const & columnInfo = this->get_ColumnInfo(index);
+//      qDebug() <<
+//         Q_FUNC_INFO << columnInfo.columnFqName << ", propertyPath:" << columnInfo.propertyPath << ", index:" <<
+//         index << ", index.column():" << index.column() << ", columnInfo.typeInfo:" << columnInfo.typeInfo;
+
+      if constexpr (HasMemberName<ColumnIndex>) {
+         if (columnIndex == ColumnIndex::Name) {
+            return defaults;
+         }
+      }
+      // If there's a TotalInventory column, it's read-only
+      if constexpr (HasMemberTotalInventory<ColumnIndex>) {
+         if (columnIndex == ColumnIndex::TotalInventory) {
+            return defaults /*| Qt::ItemIsEnabled*/;
+         }
+      }
+      // This is only for SaltTableModel and RecipeAdjustmentSaltTableModel, but it's still less work to do a special
+      // case here than have each subclass implement its own flags function.
+      if constexpr (HasMemberPctAcid<ColumnIndex>) {
+         if (columnIndex == Derived::ColumnIndex::PctAcid) {
+            bool isAcid;
+            if constexpr (std::same_as<std::remove_cvref_t<NE>, RecipeAdjustmentSalt>) {
+               isAcid = this->derived().m_rows[index.row()]->salt()->isAcid();
+            } else {
+               isAcid = this->derived().m_rows[index.row()]->isAcid();
+            }
+            if (!isAcid) {
+               return Qt::NoItemFlags;
+            }
+         }
+      }
+
+      //
+      // If the underlying property is read-only, then this column should not be editable
+      //
+      if (columnInfo.typeInfo.isReadOnly()) {
+         return defaults /*| Qt::ItemIsEnabled*/;
+      }
+
+      return defaults | (tableIsEditable ? Qt::ItemIsEditable : Qt::NoItemFlags);
+   }
+
    //! \brief Default implementation for Derived::setData
    template<bool updateHeader = false>
    bool doSetDataDefault(QModelIndex const & index, QVariant const & value, int role) {
@@ -800,56 +813,6 @@ protected:
    QList< std::shared_ptr<NE> > m_rows;
 };
 
-namespace TableModelHelper {
-
-   /**
-    * \brief Implementation for \c TableModelBase::Derived::flags
-    *
-    *        NB: This can't be a member function of \c TableModelBase because otherwise the compiler will complain about
-    *            "invalid use of incomplete type" for \c Derived.
-    */
-   template<class Derived>
-   Qt::ItemFlags doFlags(
-      QModelIndex const & index,
-      bool const editable,
-      std::vector<std::pair<typename Derived::ColumnIndex, typename Qt::ItemFlags>> const & columnExtraFlags = {},
-      Qt::ItemFlags defaults = Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled
-   ) {
-      auto const columnIndex = static_cast<Derived::ColumnIndex>(index.column());
-      if (columnIndex == Derived::ColumnIndex::Name) {
-         return defaults;
-      }
-      auto extraFlags = std::find_if(
-         columnExtraFlags.begin(),
-         columnExtraFlags.end(),
-         [&](std::pair<typename Derived::ColumnIndex, typename Qt::ItemFlags> const & element) {
-            return element.first == columnIndex;
-         }
-      );
-      if (extraFlags != columnExtraFlags.end()) {
-         return defaults | extraFlags->second;
-      }
-
-      return defaults | (editable ? Qt::ItemIsEditable : Qt::NoItemFlags);
-   }
-
-}
-
-/**
- * \brief Derived classes should include this in their header file, right before their class declaration
- *
- *        Note we have to be careful about comment formats in macro definitions
- */
-#define TABLE_MODEL_TRAITS(NeName, ...) \
-   /* You have to get the order of everything right with traits classes, but the */ \
-   /* end result is that we can refer to BoilTableModel::ColumnIndex::Name etc.  */ \
-   class NeName##TableModel;                                                        \
-   template <> struct TableModelTraits<NeName##TableModel> {                        \
-      enum class ColumnIndex {                                                      \
-         __VA_ARGS__                                                                \
-      };                                                                            \
-   };                                                                               \
-
 
 /**
  * \brief Derived classes should include this in their header file, right after Q_OBJECT
@@ -876,6 +839,12 @@ namespace TableModelHelper {
       void updateTotals();                                                                                       \
                                                                                                                  \
    public:                                                                                                       \
+      /** \brief Reimplemented from \c QAbstractTableModel */                                                    \
+      virtual int columnCount(QModelIndex const & parent = QModelIndex()) const override;                        \
+      /** \brief From \c BtTableModel */                                                                         \
+      virtual ColumnInfo columnInfo(int section) const override;                                                 \
+      /** \brief From \c BtTableModel */                                                                         \
+      virtual QVariant columnLabel(int section) const override;                                                  \
       /** \brief Reimplemented from QAbstractTableModel. */                                                      \
       virtual int rowCount(QModelIndex const & parent = QModelIndex()) const override;                           \
       /** \brief Reimplemented from QAbstractTableModel. */                                                      \
@@ -905,7 +874,7 @@ namespace TableModelHelper {
  *        NB: Mostly I have tried to make these macro-included function bodies trivial.  Macros are a bit clunky, so we
  *            only really want to use them for the things that are hard to do other ways.
  *
- *        Note that the RecipePropertyName parameter is ultimately ignored in updateInventory for table models that
+ *        Note that the RecipePropertyName parameter is ultimately ignored in updateStockPurchase for table models that
  *        do not observe a recipe, so can be set to PropertyNames::None::none in such cases.
  */
 #define TABLE_MODEL_COMMON_CODE(NeName, LcNeName, RecipePropertyName) \
@@ -916,9 +885,21 @@ namespace TableModelHelper {
    Recipe * NeName##TableModel::getObservedRecipe() const {                                             \
       return this->doGetObservedRecipe<NeName##TableModel>();                                           \
    }                                                                                                    \
+   int NeName##TableModel::columnCount([[maybe_unused]] QModelIndex const & parent) const {             \
+      return ColumnOwnerTraits<NeName##TableModel>::numColumns();                                       \
+   }                                                                                                    \
+   ColumnInfo NeName##TableModel::columnInfo(int section) const {                                       \
+      return ColumnOwnerTraits<NeName##TableModel>::getColumnInfo(section);                             \
+   }                                                                                                    \
+   QVariant NeName##TableModel::columnLabel(int section) const {                                        \
+      return ColumnOwnerTraits<NeName##TableModel>::getColumnLabel(section);                            \
+   }                                                                                                    \
    int NeName##TableModel::rowCount([[maybe_unused]] QModelIndex const & parent) const {                \
       return this->m_rows.size();                                                                       \
    }                                                                                                    \
+   Qt::ItemFlags NeName##TableModel::flags(QModelIndex const & index) const {                           \
+      return this->doFlags(index);                                                                      \
+   }                                                                                                    \
    void NeName##TableModel::addItem(int itemId) {                                                       \
       this->addById(itemId);                                                                            \
       return;                                                                                           \
@@ -933,7 +914,7 @@ namespace TableModelHelper {
    }                                                                                                    \
                                                                                                         \
    void NeName##TableModel::changedInventory(int invKey, BtStringConst const & propertyName) {          \
-      this->updateInventory<NeName##TableModel>(invKey, propertyName);                                  \
+      this->updateStockPurchase<NeName##TableModel>(invKey, propertyName);                                  \
       return;                                                                                           \
    }
 
