@@ -1,5 +1,5 @@
 /*╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
- * serialization/ImportExport.cpp is part of Brewtarget, and is copyright the following authors 2013-2025:
+ * serialization/ImportExport.cpp is part of Brewtarget, and is copyright the following authors 2013-2026:
  *   • Matt Young <mfsy@yahoo.com>
  *   • Mik Firestone <mikfire@gmail.com>
  *
@@ -44,6 +44,22 @@ namespace {
       IMPORT
    };
 
+   //
+   // Note that we cannot guarantee the file extension is at the end of the input string -- eg we might be matching
+   // against "BeerJSON format (*.json)".
+   //
+   // In the regex below, we assume:
+   //   - File extension starts with dot -- "[.]" (Saves trying to work out how many backslashes to use to escape dot!)
+   //   - File extensions are only latin letters but may be upper or lower case or both -- "[a-zA-Z]"
+   //   - File extensions are any non-zero length -- "+"
+   //   - File extension is either at end of string or followed by something that is not a letter or a
+   //     digit -- "[^a-zA-Z0-9]|$"
+   //
+   // This is a bit overkill for what we need for now, but not hugely, and is potentially useful for future expansion
+   // (eg if we want to generalise file extension handling).
+   //
+   static QRegularExpression const fileExtensionRegexp {"[.]([a-zA-Z]+)[^a-zA-Z0-9]|$"};
+
    /**
     * \brief Display a file dialog for selecting file(s) for reading / writing
     */
@@ -95,13 +111,24 @@ namespace {
             &fileChooser,
             &QFileDialog::filterSelected,
             [&fileChooser](QString const & filter) {
-               // The Qt docs are a bit silent about what the parameter is for the QFileDialog::filterSelected signal.
-               // By adding a logging statement here, we discover that we get either "*.json" or "*.xml".  So we just
-               // need to chop off the first two characters to get default suffix.
                //
-               // In Qt 6, we can replace `mid` with `sliced` which is faster apparently.
-               qDebug() << Q_FUNC_INFO << "Export filter is:" << filter;
-               fileChooser.setDefaultSuffix(filter.mid(2));
+               // The Qt docs are a bit silent about what the parameter is for the QFileDialog::filterSelected signal.
+               // By adding a logging statement here, we discovered in Qt5 that we got either "*.json" or "*.xml".
+               // However, in Qt6, we got "BeerJSON format (*.json)" or "BeerXML format (*.xml)".
+               //
+               QRegularExpressionMatch match = fileExtensionRegexp.match(filter);
+               if (!match.hasMatch()) {
+                  //
+                  // This shouldn't happen because we should be parsing one of the options we gave in the fileChooser
+                  // constructor above.
+                  //
+                  qCritical() << Q_FUNC_INFO << "Unable to parse" << filter;
+                  return;
+               }
+
+               QString const suffix = match.captured(1).toLower();
+               qDebug() << Q_FUNC_INFO << "Export filter is:" << filter << ".  From this, suffix is:" << suffix;
+               fileChooser.setDefaultSuffix(suffix);
                return;
             }
          );
@@ -112,14 +139,33 @@ namespace {
          return std::nullopt;
       }
 
+      //
+      // QFileDialog::defaultSuffix() excludes the leading dot on the suffix, but things are easier for us below if we
+      // include it.
+      //
+      QString const defaultSuffix = QString{".%1"}.arg(fileChooser.defaultSuffix());
+      QList<QString> selectedFiles = fileChooser.selectedFiles();
+
       qDebug() <<
-         Q_FUNC_INFO << "Selected" << fileChooser.selectedFiles().length() << "file(s) (from directory" <<
-         fileChooser.directory() << "):" << fileChooser.selectedFiles();
+         Q_FUNC_INFO << "Selected" << selectedFiles.length() << "file(s) (from directory" << fileChooser.directory() <<
+         "):" << fileChooser.selectedFiles() << ". Default suffix:" << defaultSuffix;
 
       // Remember the directory for next time
       fileChooserDirectory = fileChooser.directory().canonicalPath();
 
-      return fileChooser.selectedFiles();
+      //
+      // If we are writing, then, for any files that do not have a suffix, add the default one
+      //
+      if (!importing) {
+         for (QString & file : selectedFiles) {
+            if (!file.endsWith(".json", Qt::CaseInsensitive) &&
+                !file.endsWith(".xml" , Qt::CaseInsensitive)) {
+               file.append(defaultSuffix);
+            }
+         }
+      }
+
+      return selectedFiles;
    }
 
    /**
@@ -233,6 +279,8 @@ bool ImportExport::importFromFiles(std::optional<QStringList> inputFiles) {
          succeeded = BeerXML::getInstance().importFromXML(filename, userMessageAsStream);
       } else {
          qInfo() << Q_FUNC_INFO << "Don't understand file extension on" << filename << "so ignoring!";
+         userMessageAsStream <<
+            QObject::tr("Did not recognise file extension on \"%1\" so nothing written.").arg(filename);
       }
       qDebug() << Q_FUNC_INFO << "Import " << (succeeded ? "succeeded" : "failed");
       importExportMsg(ImportOrExport::IMPORT, filename, succeeded, userMessage);
@@ -245,25 +293,20 @@ bool ImportExport::importFromFiles(std::optional<QStringList> inputFiles) {
    return allSucceeded;
 }
 
-
-bool ImportExport::exportToFile(QList<Recipe      const *> const * recipes,
-                                QList<Equipment   const *> const * equipments,
-                                QList<Fermentable const *> const * fermentables,
-                                QList<Hop         const *> const * hops,
-                                QList<Misc        const *> const * miscs,
-                                QList<Style       const *> const * styles,
-                                QList<Water       const *> const * waters,
-                                QList<Yeast       const *> const * yeasts) {
+bool ImportExport::exportToFile(ImportExport::Lists const & exportLists) {
    // It's the caller's responsibility to ensure that at least one list is supplied and that at least one of the
    // supplied lists is non-empty
-   Q_ASSERT((recipes      && recipes     ->size() > 0) ||
-            (equipments   && equipments  ->size() > 0) ||
-            (fermentables && fermentables->size() > 0) ||
-            (hops         && hops        ->size() > 0) ||
-            (miscs        && miscs       ->size() > 0) ||
-            (styles       && styles      ->size() > 0) ||
-            (waters       && waters      ->size() > 0) ||
-            (yeasts       && yeasts      ->size() > 0));
+   Q_ASSERT((exportLists.recipes       && exportLists.recipes      ->size() > 0) ||
+            (exportLists.equipments    && exportLists.equipments   ->size() > 0) ||
+            (exportLists.fermentables  && exportLists.fermentables ->size() > 0) ||
+            (exportLists.hops          && exportLists.hops         ->size() > 0) ||
+            (exportLists.miscs         && exportLists.miscs        ->size() > 0) ||
+            (exportLists.styles        && exportLists.styles       ->size() > 0) ||
+            (exportLists.waters        && exportLists.waters       ->size() > 0) ||
+            (exportLists.yeasts        && exportLists.yeasts       ->size() > 0) ||
+            (exportLists.mashes        && exportLists.mashes       ->size() > 0) ||
+            (exportLists.boils         && exportLists.boils        ->size() > 0) ||
+            (exportLists.fermentations && exportLists.fermentations->size() > 0));
 
    auto selectedFiles = selectFiles(ImportOrExport::EXPORT);
    if (!selectedFiles) {
@@ -278,31 +321,35 @@ bool ImportExport::exportToFile(QList<Recipe      const *> const * recipes,
    QFile outFile;
    outFile.setFileName(filename);
 
+   bool succeeded = false;
+
    if (!outFile.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
       qWarning() << Q_FUNC_INFO << "Could not open" << filename << "for writing.";
-      return false;
-   }
+      userMessageAsStream << QObject::tr("Could not open \"%1\" for writing").arg(filename);
 
-   if (filename.endsWith("json", Qt::CaseInsensitive)) {
+   } else if (filename.endsWith(".json", Qt::CaseInsensitive)) {
       //
       // It's not strictly required by the BeerJSON standard, but we'll get a better export of Recipe if we also
       // explicitly export all the ingredients.  This is because, in BeerJSON (unlike BeerXML), the Recipe specification
       // includes only partial information about each Hop/Fermentable/etc addition.  This is fine if you already have
       // those ingredients in your database when you're reading a Recipe in, but doesn't work so well when you don't.
       //
-      QSet<Fermentable const *> setOfFermentable = makeSet(fermentables, recipes);
-      QSet<Hop         const *> setOfHop         = makeSet(hops        , recipes);
-      QSet<Misc        const *> setOfMisc        = makeSet(miscs       , recipes);
-      QSet<Yeast       const *> setOfYeast       = makeSet(yeasts      , recipes);
-      QSet<Water       const *> setOfWater       = makeSet(waters      , recipes);
+      QSet<Fermentable const *> setOfFermentable = makeSet(exportLists.fermentables, exportLists.recipes);
+      QSet<Hop         const *> setOfHop         = makeSet(exportLists.hops        , exportLists.recipes);
+      QSet<Misc        const *> setOfMisc        = makeSet(exportLists.miscs       , exportLists.recipes);
+      QSet<Yeast       const *> setOfYeast       = makeSet(exportLists.yeasts      , exportLists.recipes);
+      QSet<Water       const *> setOfWater       = makeSet(exportLists.waters      , exportLists.recipes);
       //
       // Same thing applies for Styles, and we have similar thinking for Equipments.  (Though note that, unlike in
       // BeerXML, Equipment is not part of the Recipe in BeerJSON.)
       //
-      QSet<Style       const *> setOfStyle       = makeSet(styles);
-      QSet<Equipment   const *> setOfEquipment   = makeSet(equipments);
-      if (recipes) {
-         for (Recipe const * recipe : *recipes) {
+      // In contrast, Mashes, Boils and Fermentations are fully defined when they form part of a Recipe, so we don't
+      // have to separately export them.
+      //
+      QSet<Style       const *> setOfStyle       = makeSet(exportLists.styles);
+      QSet<Equipment   const *> setOfEquipment   = makeSet(exportLists.equipments);
+      if (exportLists.recipes) {
+         for (Recipe const * recipe : *exportLists.recipes) {
             auto style     = recipe->style    (); if (style    ) { setOfStyle    .insert(style    .get()); }
             auto equipment = recipe->equipment(); if (equipment) { setOfEquipment.insert(equipment.get()); }
          }
@@ -316,13 +363,14 @@ bool ImportExport::exportToFile(QList<Recipe      const *> const * recipes,
       if (!setOfStyle      .isEmpty()   ) { exporter.add(setOfStyle      .values()); }
       if (!setOfEquipment  .isEmpty()   ) { exporter.add(setOfEquipment  .values()); }
       if (!setOfWater      .isEmpty()   ) { exporter.add(setOfWater      .values()); }
-      if (recipes && recipes->size() > 0) { exporter.add(*recipes                 ); }
+      if (exportLists.mashes        && exportLists.mashes       ->size() > 0) { exporter.add(*exportLists.mashes       ); }
+      if (exportLists.boils         && exportLists.boils        ->size() > 0) { exporter.add(*exportLists.boils        ); }
+      if (exportLists.fermentations && exportLists.fermentations->size() > 0) { exporter.add(*exportLists.fermentations); }
+      if (exportLists.recipes       && exportLists.recipes      ->size() > 0) { exporter.add(*exportLists.recipes      ); }
 
       exporter.close();
-      return true;
-   }
-
-   if (filename.endsWith("xml", Qt::CaseInsensitive)) {
+      succeeded = true;
+   } else if (filename.endsWith(".xml", Qt::CaseInsensitive)) {
       BeerXML & bxml = BeerXML::getInstance();
       // The slightly non-standard-XML format of BeerXML means the common bit (which gets written by createXmlFile) is
       // just at the start and there is no "closing" bit to write after we write all the data.
@@ -336,24 +384,47 @@ bool ImportExport::exportToFile(QList<Recipe      const *> const * recipes,
       //    MISCS
       //    WATERS
       //    STYLES
-      //    MASH_STEPS
-      //    MASHS
+      //    MASHS [sic] (inside of which are MASH_STEPS)
       //    RECIPES
       //    EQUIPMENTS
       //
-      if (hops         && hops        ->size() > 0) { bxml.toXml(*hops,         outFile); }
-      if (fermentables && fermentables->size() > 0) { bxml.toXml(*fermentables, outFile); }
-      if (yeasts       && yeasts      ->size() > 0) { bxml.toXml(*yeasts,       outFile); }
-      if (miscs        && miscs       ->size() > 0) { bxml.toXml(*miscs,        outFile); }
-      if (waters       && waters      ->size() > 0) { bxml.toXml(*waters,       outFile); }
-      if (styles       && styles      ->size() > 0) { bxml.toXml(*styles,       outFile); }
-      if (recipes      && recipes     ->size() > 0) { bxml.toXml(*recipes,      outFile); }
-      if (equipments   && equipments  ->size() > 0) { bxml.toXml(*equipments,   outFile); }
+      if (exportLists.hops          && exportLists.hops         ->size() > 0) { bxml.toXml(*exportLists.hops        , outFile); }
+      if (exportLists.fermentables  && exportLists.fermentables ->size() > 0) { bxml.toXml(*exportLists.fermentables, outFile); }
+      if (exportLists.yeasts        && exportLists.yeasts       ->size() > 0) { bxml.toXml(*exportLists.yeasts      , outFile); }
+      if (exportLists.miscs         && exportLists.miscs        ->size() > 0) { bxml.toXml(*exportLists.miscs       , outFile); }
+      if (exportLists.waters        && exportLists.waters       ->size() > 0) { bxml.toXml(*exportLists.waters      , outFile); }
+      if (exportLists.styles        && exportLists.styles       ->size() > 0) { bxml.toXml(*exportLists.styles      , outFile); }
+      if (exportLists.mashes        && exportLists.mashes       ->size() > 0) { bxml.toXml(*exportLists.mashes      , outFile); }
+      if (exportLists.boils         && exportLists.boils        ->size() > 0) { bxml.toXml(*exportLists.mashes      , outFile); }
+      if (exportLists.fermentations && exportLists.fermentations->size() > 0) { bxml.toXml(*exportLists.mashes      , outFile); }
+      if (exportLists.recipes       && exportLists.recipes      ->size() > 0) { bxml.toXml(*exportLists.recipes     , outFile); }
+      if (exportLists.equipments    && exportLists.equipments   ->size() > 0) { bxml.toXml(*exportLists.equipments  , outFile); }
 
-      return true;
+      succeeded = true;
+   } else {
+      qInfo() << Q_FUNC_INFO << "Don't understand file extension on" << filename << "so ignoring!";
+      userMessageAsStream <<
+         QObject::tr("Did not recognise file extension on \"%1\" so nothing read.").arg(filename);
    }
 
-   qInfo() << Q_FUNC_INFO << "Don't understand file extension on" << filename << "so ignoring!";
+
+   qDebug() << Q_FUNC_INFO << "Export" << (succeeded ? "succeeded" : "failed");
+   importExportMsg(ImportOrExport::EXPORT, filename, succeeded, userMessage);
 
    return false;
 }
+
+//
+// This is a bit clunky, but it works!
+//
+template<> bool ImportExport::exportToFile(QList<Recipe       const *> const & items) { Lists exportLists{.recipes       = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Equipment    const *> const & items) { Lists exportLists{.equipments    = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Fermentable  const *> const & items) { Lists exportLists{.fermentables  = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Hop          const *> const & items) { Lists exportLists{.hops          = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Misc         const *> const & items) { Lists exportLists{.miscs         = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Style        const *> const & items) { Lists exportLists{.styles        = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Water        const *> const & items) { Lists exportLists{.waters        = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Yeast        const *> const & items) { Lists exportLists{.yeasts        = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Mash         const *> const & items) { Lists exportLists{.mashes        = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Boil         const *> const & items) { Lists exportLists{.boils         = &items}; return exportToFile(exportLists); }
+template<> bool ImportExport::exportToFile(QList<Fermentation const *> const & items) { Lists exportLists{.fermentations = &items}; return exportToFile(exportLists); }
